// #############################################################
// Module Imports
// #############################################################
const axios = require('axios');

// #############################################################
// Variables
// #############################################################
const baseUrl = "https://tenor.googleapis.com/v2/search?";

class Request {
    constructor() {
        this.completed = false;
    }

    get(url) {
        return new Promise(async (resolve, reject) => {
            axios.get(url).then((response) => {
                this.completed = true;
                resolve(response.data);
            }).catch((error) => {
                this.completed = true;
                reject(error);
            });
        });
    }
}

class Tenor {
    constructor(tenorApiKey) {
        this.token = tenorApiKey;
        return this;
    }

    async configure() {
        return new Promise((resolve, reject) => {
            if (!this.token) {
                return reject(new Error('No API key provided'));
            }

            this.search('cat', 1).then((gifs) => { // Search with a valid key
                if (gifs.length !== 1) { // Check for a valid response
                    return reject(new Error('Unexpected response from known-good API query.'));
                } else { // Key is valid
                    resolve('Tenor API key is valid and API is working correctly.');
                }
            }).catch((error) => {
                if (error.response && error.response.status === 401) {
                    return reject(new Error(`Invalid API Key`));
                } else {
                    return reject(error);
                }
            });
        });
    }

    search(query, options) {
        return new Promise((resolve, reject) => {
            // Form the search url
            let searchUrl = baseUrl + `q=${query}`;
            searchUrl += `&key=${this.token}`;
            if (options) {
                if (typeof options === 'number') {
                    searchUrl += `&limit=${options}`;
                } else if (typeof options === 'object') {
                    if (options.limit) searchUrl += `&limit=${options.limit}`; // Default 20 max 50
                    if (options.clientKey) searchUrl += `&client_key=${options.clientKey}`;
                    if (options.random) searchUrl += `&random=${options.random}`;
                    if (options.position) searchUrl += `&pos=${options.position}`;
                    if (options.country) searchUrl += `&country=${options.country}`;
                    if (options.locale) searchUrl += `&locale=${options.locale}`;
                    if (options.mediaFilter) searchUrl += `&media_filter=${options.mediaFilter}`;
                    if (options.contentFilter) searchUrl += `&contentfilter=${options.contentFilter}`;
                }
            }
            // Submit the search
            new Request().get(searchUrl).then((response) => {
                // Validate the results
                if (!response.results || response.results.length === 0) {
                    return reject(new Error('No results found'));
                }

                // Create object to store results
                const gifs = [];
                // Manipulate the results
                for (const gif of response.results) {
                    gifs.push(new Gif(gif));
                }

                // Return the results
                resolve(gifs);
            }).catch(error => {
                if (error.response.status >= 400 && error.response.status <= 499) {
                    return reject(new Error(`Invalid API Key Provided: ${this.token}`));
                }
                return reject(error);
            });
        });
    }
}

class Gif {
    constructor(gif) {
        this.url = gif.url;
        this.id = gif.id;
        this.created = gif.created;
        this.mediaUrl = gif.media_formats.gif.url;
        return this;
    }
}

module.exports = Tenor;